@@ -33,290 +33,8 @@
 /* We always have a toplevel entry for P_ROOT */
 #define P_ROOT   "augeas"
 
-#define L_BRACK '['
-#define R_BRACK ']'
-
-static const char *const last_func = "[last()]";
-
 #define TREE_HIDDEN(tree) ((tree)->label == NULL)
 
-/* Internal representation of a path in the tree */
-struct segment {
-    int          index;
-    unsigned int fixed : 1; /* Match only one index (either INDEX or LAST) */
-    unsigned int last  : 1; /* Match the last node with LABEL */
-    unsigned int any   : 1; /* Match any node with any label, implies !FIXED */
-    char        *label;
-    struct tree *tree;
-};
-
-struct path {
-    size_t      nsegments;
-    struct segment *segments;
-    struct tree    *root;
-};
-
-#define for_each_segment(seg, path)                                 \
-    for (typeof(path->segments) (seg) = (path)->segments;           \
-         (seg) < (path)->segments + (path)->nsegments;              \
-         (seg)++)
-
-#define last_segment(path) (path->segments + path->nsegments - 1)
-
-static int xstrtoul(const char *nptr, char **endptr, int base) {
-    unsigned long val;
-    char *end;
-    int result;
-
-    errno = 0;
-    val = strtoul(nptr, &end, base);
-    if (errno || (!endptr && *end) || end == nptr || (int) val != val) {
-        result = -1;
-    } else {
-        result = val;
-    }
-    if (endptr)
-        *endptr = end;
-    return result;
-}
-
-static int sibling_index(struct tree *siblings, struct tree *tree) {
-    int indx = 0;
-    list_for_each(t, siblings) {
-        if (streqv(t->label, tree->label))
-            indx += 1;
-        if (t == tree)
-            return indx;
-    }
-    return indx;
-}
-
-static void free_path(struct path *path) {
-    if (path == NULL)
-        return;
-
-    if (path->segments != NULL) {
-        for (int i=0; i < path->nsegments; i++) {
-            free(path->segments[i].label);
-        }
-        free(path->segments);
-    }
-    free(path);
-}
-
-/* Return the start of the list of siblings of SEG->TREE */
-static struct tree *seg_siblings(struct path *path, struct segment *seg) {
-    return (seg == path->segments) ? path->root : (seg-1)->tree->children;
-}
-
-/* Return the parent of SEG->TREE or NULL if SEG->TREE is on the toplevel */
-static struct tree *seg_parent(struct path *path, struct segment *seg) {
-    return (seg == path->segments) ? NULL : (seg-1)->tree;
-}
-
-/* Take a path expression PATH and turn it into a path structure usable for
- * search. The TREE components of the PATH segments will be NULL. Call
- * PATH_FIRST to initialize them to the first match.
- *
- * A path expression follows the grammar
- * PATH = '/' ? SEGMENT ( '/' SEGMENT )* '/' ?
- * SEGMENT = STRING ('[' N ']') ? | '*'
- * where STRING is any string not containing '/' and N is a positive number
- */
-static struct path *make_path(const struct tree *root, const char *path) {
-    struct path *result;
-
-    CALLOC(result, 1);
-    if (result == NULL)
-        return NULL;
-    result->root = (struct tree *) root;
-
-    if (*path != SEP)
-        result->nsegments = 1;
-    for (const char *p = path; *p != '\0'; p++) {
-        if (*p == SEP) {
-            while (*p == SEP) p++;
-            if (*p == '\0')
-                break;
-            result->nsegments++;
-        }
-    }
-    if (result->nsegments == 0)
-        goto error;
-
-    CALLOC(result->segments, result->nsegments);
-    if (result->segments == NULL)
-        goto error;
-
-    for_each_segment(seg, result) {
-        const char *next, *brack;
-
-        while (*path == SEP)
-            path += 1;
-        assert(*path);
-
-        brack = strchr(path, L_BRACK);
-        next = strchr(path, SEP);
-        if (next == NULL)
-            next = path + strlen(path);
-
-        if (path[0] == '*') {
-            seg->any = 1;
-        } else if (brack == NULL || brack >= next) {
-            seg->index = 0;
-            seg->label = strndup(path, next - path);
-        } else {
-            seg->label = strndup(path, brack - path);
-            if (STREQLEN(brack, last_func, strlen(last_func))) {
-                seg->last = 1;
-                seg->fixed = 1;
-            } else {
-                char *end;
-                seg->index = xstrtoul(brack + 1, &end, 10);
-                seg->fixed = 1;
-                if (seg->index <= 0)
-                    goto error;
-                if (*end != R_BRACK)
-                    goto error;
-            }
-        }
-        path = next;
-        while (*path == SEP) path += 1;
-    }
-    return result;
-
- error:
-    free_path(result);
-    return NULL;
-}
-
-static int complete_path(struct path *path, int segnr, struct tree *tree) {
-    if (segnr == path->nsegments)
-        return 1;
-
-    if (tree == NULL)
-        return 0;
-
-    int cur = segnr;
-    path->segments[cur].tree = tree;
-
-    while (1) {
-        struct segment *seg = path->segments + cur;
-        int found = 0;
-        struct tree *siblings = seg_siblings(path, seg);
-        list_for_each(t, seg->tree) {
-            if (seg->any || streqv(t->label, seg->label)) {
-                int indx = sibling_index(siblings, t);
-                if (!seg->fixed || seg->last || indx == seg->index) {
-                    found = 1;
-                    seg->tree = t;
-                    seg->index = indx;
-                    if (!seg->last)
-                        break;
-                }
-            }
-        }
-        if (found) {
-            cur += 1;
-            if (cur == path->nsegments)
-                break;
-            path->segments[cur].tree = seg->tree->children;
-        } else {
-            cur -= 1;
-            if (cur < segnr)
-                break;
-            path->segments[cur].tree = path->segments[cur].tree->next;
-        }
-    }
-    return (cur == path->nsegments);
-}
-
-static struct tree *path_next(struct path *path) {
-    for (int i = path->nsegments-1; i >= 0; i--) {
-        struct segment *seg = path->segments + i;
-        struct tree *siblings = seg_siblings(path, seg);
-        if (! seg->fixed) {
-            list_for_each(t, seg->tree->next) {
-                if (seg->any || streqv(t->label, seg->label)) {
-                    seg->index += 1;
-                    if (complete_path(path, i+1, t->children)) {
-                        seg->tree = t;
-                        if (seg->any)
-                            seg->index = sibling_index(siblings, t);
-                        return last_segment(path)->tree;
-                    }
-                }
-            }
-        }
-    }
-    return NULL;
-}
-
-/* Find the first node in TREE matching PATH. Fill in the TREE pointers along
- * the way
- */
-static struct tree *path_first(struct path *path) {
-    struct tree *tree = path->root;
-    struct segment *seg = path->segments;
-    int indx = 0;
-    int found = 0;
-
-    seg->tree = NULL;
-    list_for_each(t, tree) {
-        if (seg->any || streqv(t->label, seg->label)) {
-            indx += 1;
-            if (seg->last || !seg->fixed || indx == seg->index) {
-                seg->tree = t;
-                seg->index = indx;
-                if (seg->any)
-                    seg->index = sibling_index(tree, t);
-                if (!seg->last && complete_path(path, 1, t->children)) {
-                    found = 1;
-                    break;
-                }
-            }
-        }
-    }
-    if (seg->last && seg->tree && complete_path(path, 1, seg->tree->children))
-        found = 1;
-    return found ? last_segment(path)->tree : NULL;
-}
-
-/* Fill the TREE pointer in PATH with a longest prefix that matches in the
- * tree. PATH can not contain any wildcards, or other constructs that would
- * allow more than one path in the tree to match.
- *
- * Return 1 if a node was found that exactly matches PATH, 0 if an incomplete
- * prefix matches, and -1 if more than one node in the tree match
- */
-static int path_find_one(struct path *path) {
-    struct tree *tree = path->root;
-
-    for_each_segment(seg, path) {
-        int indx = 0;
-        if (seg->any)
-            return -1;
-
-        seg->tree = NULL;
-        list_for_each(t, tree) {
-            if (streqv(t->label, seg->label)) {
-                indx += 1;
-                if (!seg->fixed && seg->tree != NULL)
-                    return -1;
-
-                if (!seg->fixed || seg->last || indx == seg->index) {
-                    seg->tree = t;
-                    seg->index = indx;
-                }
-            }
-        }
-        if (seg->tree == NULL)
-            return 0;
-        tree = seg->tree->children;
-    }
-    return 1;
-}
-
 static const char *pretty_label(const struct tree *tree) {
     if (tree == NULL)
         return "(no_tree)";
@@ -326,73 +44,26 @@ static const char *pretty_label(const struct tree *tree) {
         return tree->label;
 }
 
-static const char *seg_label(struct segment *seg) {
-    if (seg->label != NULL)
-        return seg->label;
-    return pretty_label(seg->tree);
-}
+static char *path_expand(struct tree *tree, const char *ppath) {
+    struct tree *siblings = tree->parent->children;
 
-static int seg_needs_qual(struct path *path, struct segment *seg) {
-    struct tree *siblings;
-
-    if (seg->index > 1)
-        return 1;
-    siblings = (seg == path->segments) ? path->root : (seg-1)->tree->children;
-    list_for_each(t, siblings) {
-        if (t != seg->tree && streqv(t->label, seg->tree->label))
-            return 1;
-    }
-    return 0;
-}
-
-static char *format_path(struct path *path) {
-    size_t size = 0, used = 0;
-    char *result;
-
-    for_each_segment(seg, path) {
-        const char *label = seg_label(seg);
-        if (seg_needs_qual(path, seg)) {
-            size += snprintf(NULL, 0, "/%s[%d]", label, seg->index);
-        } else {
-            size += strlen(label) + 1;
-        }
-    }
-    size += 1;
-
-    CALLOC(result, size);
-    if (result == NULL)
-        return NULL;
-    for_each_segment(seg, path) {
-        const char *label = seg_label(seg);
-        if (seg_needs_qual(path, seg)) {
-            used += snprintf(result + used, size-used,
-                             "/%s[%d]", label, seg->index);
-        } else {
-            used += snprintf(result + used, size-used, "/%s", label);
-        }
-    }
-    return result;
-}
-
-static char *path_expand(struct tree *tree, struct tree *start,
-                         const char *ppath) {
     char *path;
     const char *label;
-    int cnt = 0, ind = 0, inc = 1, r;
+    int cnt = 0, ind = 0, r;
 
-    list_for_each(t, start) {
+    list_for_each(t, siblings) {
         if (streqv(t->label, tree->label)) {
             cnt += 1;
-            ind += inc;
             if (t == tree)
-                inc = 0;
+                ind = cnt;
         }
     }
-    if (cnt == 1)
-        ind = 0;
+
+    if (ppath == NULL)
+        ppath = "";
 
     label = pretty_label(tree);
-    if (ind > 0) {
+    if (cnt > 1) {
         r = asprintf(&path, "%s/%s[%d]", ppath, label, ind);
     } else {
         r = asprintf(&path, "%s/%s", ppath, label);
@@ -402,43 +73,24 @@ static char *path_expand(struct tree *tree, struct tree *start,
     return path;
 }
 
-/* Expand the tree ROOT so that it contains all components of PATH. PATH
- * must have been initialized against ROOT by a call to PATH_FIND_ONE.
- *
- * Return the first segment that was created by this operation, or NULL on
- * error.
- */
-static struct segment *tree_create(struct tree *root, struct path *path) {
-    struct segment *s, *seg;
+static char *format_path(struct tree *tree) {
+    int depth, i;
+    struct tree *t, **anc;
+    char *path = NULL;
 
-    for (seg = path->segments;
-         seg->tree != NULL && seg <= last_segment(path);
-         seg++);
-    if (seg->tree != NULL)
+    for (t = tree, depth = 1; ! ROOT_P(t); depth++, t = t->parent);
+    if (ALLOC_N(anc, depth) < 0)
         return NULL;
 
-    for (s = seg ; s <= last_segment(path); s++) {
-        s->tree = make_tree(strdup(s->label), NULL, NULL);
-        if (s->tree == NULL || s->tree->label == NULL)
-            goto error;
-    }
-    for (s = seg ; s < last_segment(path); s++) {
-        s->tree->children = (s+1)->tree;
-    }
+    for (t = tree, i = depth - 1; i >= 0; i--, t = t->parent)
+        anc[i] = t;
 
-    struct tree *tree = seg_parent(path, seg);
-    if (tree == NULL)
-        list_append(root, seg->tree);
-    else
-        list_append(tree->children, seg->tree);
-
-    return seg;
- error:
-    for (s = seg; s->tree != NULL && s <= last_segment(path); s++) {
-        free_tree(s->tree);
-        s->tree = NULL;
+    for (i = 0; i < depth; i++) {
+        char *p = path_expand(anc[i], path);
+        free(path);
+        path = p;
     }
-    return NULL;
+    return path;
 }
 
 /* Propagate dirty flags towards the root */
@@ -479,15 +131,23 @@ static const char *init_root(const char *root0) {
 struct augeas *aug_init(const char *root, const char *loadpath,
                         unsigned int flags) {
     struct augeas *result;
+    struct tree *tree_root = make_tree(NULL, NULL, NULL, NULL);
+
+    if (tree_root == NULL)
+        return NULL;
 
     CALLOC(result, 1);
-    result->tree = make_tree(NULL, NULL, NULL);
+    result->origin = make_tree_origin(tree_root);
+    if (result->origin == NULL) {
+        free_tree(tree_root);
+        goto error;
+    }
 
     result->flags = flags;
 
     result->root = init_root(root);
 
-    result->tree->label = strdup(P_ROOT);
+    result->origin->children->label = strdup(P_ROOT);
 
     result->modpathz = NULL;
     result->nmodpath = 0;
@@ -499,28 +159,32 @@ struct augeas *aug_init(const char *root, const char *loadpath,
     if (env != NULL) {
         argz_add_sep(&result->modpathz, &result->nmodpath, env, PATH_SEP_CHAR);
     }
-    argz_add(&result->modpathz, &result->nmodpath, AUGEAS_LENS_DIR);
+    if (!(flags & AUG_NO_STDINC)) {
+        argz_add(&result->modpathz, &result->nmodpath, AUGEAS_LENS_DIR);
+    }
     /* Clean up trailing slashes */
-    argz_stringify(result->modpathz, result->nmodpath, PATH_SEP_CHAR);
-    char *s, *t;
-    for (s = result->modpathz, t = result->modpathz; *s != '\0'; s++) {
-        char *p = s;
-        if (*p == '/') {
-            while (*p == '/') p += 1;
-            if (*p == '\0' || *p == PATH_SEP_CHAR)
-                s = p;
+    if (result->nmodpath > 0) {
+        argz_stringify(result->modpathz, result->nmodpath, PATH_SEP_CHAR);
+        char *s, *t;
+        for (s = result->modpathz, t = result->modpathz; *s != '\0'; s++) {
+            char *p = s;
+            if (*p == '/') {
+                while (*p == '/') p += 1;
+                if (*p == '\0' || *p == PATH_SEP_CHAR)
+                    s = p;
+            }
+            if (t != s)
+                *t++ = *s;
+            else
+                t += 1;
         }
-        if (t != s)
-            *t++ = *s;
-        else
-            t += 1;
+        if (t != s) {
+            *t = '\0';
+        }
+        s = result->modpathz;
+        argz_create_sep(s, PATH_SEP_CHAR, &result->modpathz, &result->nmodpath);
+        free(s);
     }
-    if (t != s) {
-        *t = '\0';
-    }
-    s = result->modpathz;
-    argz_create_sep(s, PATH_SEP_CHAR, &result->modpathz, &result->nmodpath);
-    free(s);
 
     /* We report the root dir in AUGEAS_META_ROOT, but we only use the
        value we store internally, to avoid any problems with
@@ -544,7 +208,7 @@ struct augeas *aug_init(const char *root, const char *loadpath,
             continue;
         transform_load(result, xform);
     }
-    list_for_each(tree, result->tree) {
+    list_for_each(tree, result->origin->children) {
         tree_clean(tree);
     }
     return result;
@@ -555,7 +219,8 @@ struct augeas *aug_init(const char *root, const char *loadpath,
 }
 
 int aug_get(const struct augeas *aug, const char *path, const char **value) {
-    struct path *p = make_path(aug->tree, path);
+    struct path *p = make_path(aug->origin->children, path);
+    struct tree *match;
     int r;
 
     if (p == NULL)
@@ -564,9 +229,9 @@ int aug_get(const struct augeas *aug, const char *path, const char **value) {
     if (value != NULL)
         *value = NULL;
 
-    r = path_find_one(p);
+    r = path_find_one(p, &match);
     if (r == 1 && value != NULL)
-        *value = last_segment(p)->tree->value;
+        *value = match->value;
     free_path(p);
 
     return r;
@@ -580,15 +245,9 @@ struct tree *tree_set(struct tree *root, const char *path, const char *value) {
     if (p == NULL)
         goto error;
 
-    r = path_find_one(p);
+    r = path_expand_tree(p, &tree);
     if (r == -1)
         goto error;
-
-    if (r == 0) {
-        if (tree_create(root, p) == NULL)
-            goto error;
-    }
-    tree = last_segment(p)->tree;
     free_path(p);
 
     if (tree->value != NULL) {
@@ -608,39 +267,35 @@ struct tree *tree_set(struct tree *root, const char *path, const char *value) {
 }
 
 int aug_set(struct augeas *aug, const char *path, const char *value) {
-    return tree_set(aug->tree, path, value) == NULL ? -1 : 0;
+    return tree_set(aug->origin->children, path, value) == NULL ? -1 : 0;
 }
 
-int tree_insert(struct tree **tree, const char *path, const char *label,
+int tree_insert(struct tree *origin, const char *path, const char *label,
                 int before) {
+    assert(origin->parent == origin);
+
     struct path *p = NULL;
-    struct tree *new = NULL;
+    struct tree *new = NULL, *match;
 
     if (strchr(label, SEP) != NULL)
         return -1;
 
-    p = make_path(*tree, path);
+    p = make_path(origin->children, path);
     if (p == NULL)
         goto error;
 
-    if (path_find_one(p) != 1)
+    if (path_find_one(p, &match) != 1)
         goto error;
 
-    new = make_tree(strdup(label), NULL, NULL);
+    new = make_tree(strdup(label), NULL, match->parent, NULL);
     if (new == NULL || new->label == NULL)
         goto error;
 
-    struct segment *seg = last_segment(p);
     if (before) {
-        if (seg == p->segments) {
-            list_insert_before(new, seg->tree, *tree);
-        } else {
-            struct tree *parent = seg_parent(p, seg);
-            list_insert_before(new, seg->tree, parent->children);
-        }
+        list_insert_before(new, match, new->parent->children);
     } else {
-        new->next = seg->tree->next;
-        seg->tree->next = new;
+        new->next = match->next;
+        match->next = new;
     }
     free_path(p);
     return 0;
@@ -652,10 +307,11 @@ int tree_insert(struct tree **tree, const char *path, const char *label,
 
 int aug_insert(struct augeas *aug, const char *path, const char *label,
                int before) {
-    return tree_insert(&(aug->tree), path, label, before);
+    return tree_insert(aug->origin, path, label, before);
 }
 
-struct tree *make_tree(char *label, char *value, struct tree *children) {
+struct tree *make_tree(char *label, char *value, struct tree *parent,
+                       struct tree *children) {
     struct tree *tree;
     CALLOC(tree, 1);
     if (tree == NULL)
@@ -663,11 +319,25 @@ struct tree *make_tree(char *label, char *value, struct tree *children) {
 
     tree->label = label;
     tree->value = value;
+    tree->parent = parent;
     tree->children = children;
+    list_for_each(c, tree->children)
+        c->parent = tree;
     tree->dirty = 1;
     return tree;
 }
 
+struct tree *make_tree_origin(struct tree *root) {
+    struct tree *origin = NULL;
+
+    origin = make_tree(NULL, NULL, NULL, root);
+    if (origin == NULL)
+        return NULL;
+
+    origin->parent = origin;
+    return origin;
+}
+
 /* Free one tree node */
 static void free_tree_node(struct tree *tree) {
     if (tree == NULL)
@@ -693,18 +363,20 @@ int free_tree(struct tree *tree) {
     return cnt;
 }
 
-int tree_rm(struct tree **htree, const char *path) {
+int tree_rm(struct tree *origin, const char *path) {
+    assert(origin->parent == origin);
+    assert(origin->next == NULL);
+
     struct path *p = NULL;
-    struct tree *tree, **del, **parents;
+    struct tree *root = origin->children;
+    struct tree *tree, **del;
     int cnt = 0, ndel = 0, i;
 
-    p = make_path(*htree, path);
+    p = make_path(root, path);
     if (p == NULL)
         return -1;
 
-    struct segment *seg = last_segment(p);
-
-    for (tree = path_first(p); tree != NULL; tree = path_next(p)) {
+    for (tree = path_first(p); tree != NULL; tree = path_next(p, tree)) {
         if (! TREE_HIDDEN(tree))
             ndel += 1;
     }
@@ -714,43 +386,33 @@ int tree_rm(struct tree **htree, const char *path) {
         return 0;
     }
 
-    CALLOC(del, ndel);
-    CALLOC(parents, ndel);
-    if (del == NULL || parents == NULL) {
+    if (ALLOC_N(del, ndel) < 0) {
         free(del);
-        free(parents);
         return -1;
     }
 
-    for (i = 0, tree = path_first(p); tree != NULL; tree = path_next(p)) {
+    for (i = 0, tree = path_first(p); tree != NULL; tree = path_next(p, tree)) {
         if (TREE_HIDDEN(tree))
             continue;
-        parents[i] = seg_parent(p, seg);
-        del[i] = seg->tree;
+        del[i] = tree;
         i += 1;
     }
     free_path(p);
 
     for (i = 0; i < ndel; i++) {
-        if (parents[i] == NULL) {
-            list_remove(del[i], *htree);
-            if (*htree)
-                (*htree)->dirty = 1;
-        } else {
-            list_remove(del[i], parents[i]->children);
-            parents[i]->dirty = 1;
-        }
+        assert (del[i]->parent != NULL);
+        list_remove(del[i], del[i]->parent->children);
+        del[i]->parent->dirty = 1;
         cnt += free_tree(del[i]->children) + 1;
         free_tree_node(del[i]);
     }
     free(del);
-    free(parents);
 
     return cnt;
 }
 
 int aug_rm(struct augeas *aug, const char *path) {
-    return tree_rm(&aug->tree, path);
+    return tree_rm(aug->origin, path);
 }
 
 int aug_tree_replace(struct augeas *aug, const char *path, struct tree *sub) {
@@ -761,18 +423,21 @@ int aug_tree_replace(struct augeas *aug, const char *path, struct tree *sub) {
     if (r == -1)
         goto error;
 
-    parent = tree_set(aug->tree, path, NULL);
+    parent = tree_set(aug->origin->children, path, NULL);
     if (parent == NULL)
         goto error;
 
     list_append(parent->children, sub);
+    list_for_each(s, sub) {
+        s->parent = parent;
+    }
     return 0;
  error:
     return -1;
 }
 
 int aug_mv(struct augeas *aug, const char *src, const char *dst) {
-    struct tree *root = aug->tree;
+    struct tree *root = aug->origin->children;
     struct path *s = make_path(root, src);
     struct path *d = make_path(root, dst);
     struct tree *ts, *td, *t;
@@ -782,30 +447,28 @@ int aug_mv(struct augeas *aug, const char *src, const char *dst) {
     if (s == NULL || d == NULL)
         goto done;
 
-    r = path_find_one(s);
+    r = path_find_one(s, &ts);
     if (r != 1)
         goto done;
 
-    r = path_find_one(d);
+    r = path_expand_tree(d, &td);
     if (r == -1)
         goto done;
 
-    if (r == 0) {
-        if (tree_create(root, d) == NULL)
+    /* Don't move SRC into its own descendent */
+    t = td;
+    do {
+        if (t == ts)
             goto done;
-    }
+        t = t->parent;
+    } while (! ROOT_P(t));
 
-    ts = last_segment(s)->tree;
-    for_each_segment(ds, d) {
-        /* Don't move SRC into its own descendent */
-        if (ds->tree == ts)
-            goto done;
-    }
-    td = last_segment(d)->tree;
     free_tree(td->children);
 
     td->children = ts->children;
-
+    list_for_each(c, td->children) {
+        c->parent = td;
+    }
     free(td->value);
     td->value = ts->value;
 
@@ -813,11 +476,9 @@ int aug_mv(struct augeas *aug, const char *src, const char *dst) {
     ts->children = NULL;
 
 
-    t = seg_parent(s, last_segment(s));
-    if (t == NULL) t = root;
-    list_remove(ts, t->children);
+    list_remove(ts, ts->parent->children);
 
-    t->dirty = 1;
+    ts->parent->dirty = 1;
     td->dirty = 1;
 
     free_tree(ts);
@@ -841,11 +502,11 @@ int aug_match(const struct augeas *aug, const char *pathin, char ***matches) {
         pathin = "/*";
     }
 
-    p = make_path(aug->tree, pathin);
+    p = make_path(aug->origin->children, pathin);
     if (p == NULL)
         return -1;
 
-    for (tree = path_first(p); tree != NULL; tree = path_next(p)) {
+    for (tree = path_first(p); tree != NULL; tree = path_next(p, tree)) {
         if (! TREE_HIDDEN(tree))
             cnt += 1;
     }
@@ -859,12 +520,12 @@ int aug_match(const struct augeas *aug, const char *pathin, char ***matches) {
     if (*matches == NULL)
         goto error;
 
-    p = make_path(aug->tree, pathin);
+    p = make_path(aug->origin->children, pathin);
     int i = 0;
-    for (tree = path_first(p); tree != NULL; tree = path_next(p)) {
+    for (tree = path_first(p); tree != NULL; tree = path_next(p, tree)) {
         if (TREE_HIDDEN(tree))
             continue;
-        (*matches)[i] = format_path(p);
+        (*matches)[i] = format_path(tree);
         if ((*matches)[i] == NULL) {
             goto error;
         }
@@ -885,12 +546,15 @@ int aug_match(const struct augeas *aug, const char *pathin, char ***matches) {
     return -1;
 }
 
-static int tree_save(struct augeas *aug, struct tree *tree, const char *path) {
+static int tree_save(struct augeas *aug, struct tree *tree, const char *path,
+                     int *count) {
     int result = 0;
     // FIXME: We need to detect subtrees that aren't saved by anything
+    aug_rm(aug, AUGEAS_EVENTS_SAVED);
+
     list_for_each(t, tree) {
         if (t->dirty) {
-            char *tpath;
+            char *tpath = NULL;
             struct transform *transform = NULL;
             if (asprintf(&tpath, "%s/%s", path, t->label) == -1) {
                 result = -1;
@@ -910,10 +574,26 @@ static int tree_save(struct augeas *aug, struct tree *tree, const char *path) {
                 }
             }
             if (transform != NULL) {
-                if (transform_save(aug, transform, tpath, t) == -1)
+                int r = transform_save(aug, transform, tpath, t);
+                if (r == -1)
                     result = -1;
+                else if (r > 0) {
+                    char *sp;
+                    *count += 1;
+                    if (asprintf(&sp, AUGEAS_EVENTS_SAVED "[%d]",
+                                 *count) < 0) {
+                        free(tpath);
+                        return -1;
+                    }
+                    if (aug_set(aug, sp, tpath) < 0) {
+                        free(tpath);
+                        free(sp);
+                        return -1;
+                    }
+                    free(sp);
+                }
             } else {
-                if (tree_save(aug, t->children, tpath) == -1)
+                if (tree_save(aug, t->children, tpath, count) == -1)
                     result = -1;
             }
             free(tpath);
@@ -925,25 +605,25 @@ static int tree_save(struct augeas *aug, struct tree *tree, const char *path) {
 int aug_save(struct augeas *aug) {
     int ret = 0;
     struct tree *files;
-    struct path *p = make_path(aug->tree, AUGEAS_FILES_TREE);
+    struct path *p = make_path(aug->origin->children, AUGEAS_FILES_TREE);
 
-    if (p == NULL || path_find_one(p) != 1) {
+    if (p == NULL || path_find_one(p, &files) != 1) {
         free_path(p);
         return -1;
     }
-    files = last_segment(p)->tree;
     free_path(p);
 
-    list_for_each(t, aug->tree) {
+    list_for_each(t, aug->origin->children) {
         tree_propagate_dirty(t);
     }
     if (files->dirty) {
+        int count = 0;
         list_for_each(t, files->children) {
-            if (tree_save(aug, t, AUGEAS_FILES_TREE) == -1)
+            if (tree_save(aug, t, AUGEAS_FILES_TREE, &count) == -1)
                 ret = -1;
         }
     }
-    tree_clean(aug->tree);
+    tree_clean(aug->origin->children);
     return ret;
 }
 
@@ -976,7 +656,7 @@ static int print_rec(FILE *out, struct tree *start, const char *ppath,
         if (TREE_HIDDEN(tree) && ! pr_hidden)
             continue;
 
-        path = path_expand(tree, start, ppath);
+        path = path_expand(tree, ppath);
         if (path == NULL)
             goto error;
 
@@ -1006,11 +686,11 @@ int print_tree(const struct tree *start, FILE *out, const char *pathin,
     if (p == NULL)
         return -1;
 
-    for (tree = path_first(p); tree != NULL; tree = path_next(p)) {
+    for (tree = path_first(p); tree != NULL; tree = path_next(p, tree)) {
         if (TREE_HIDDEN(tree) && ! pr_hidden)
             continue;
 
-        path = format_path(p);
+        path = format_path(tree);
         if (path == NULL)
             goto error;
         r = print_one(out, path, tree->value);
@@ -1033,13 +713,13 @@ int aug_print(const struct augeas *aug, FILE *out, const char *pathin) {
     if (pathin == NULL || strlen(pathin) == 0) {
         pathin = "/*";
     }
-    return print_tree(aug->tree, out, pathin, 0);
+    return print_tree(aug->origin->children, out, pathin, 0);
 }
 
 void aug_close(struct augeas *aug) {
     if (aug == NULL)
         return;
-    free_tree(aug->tree);
+    free_tree(aug->origin);
     unref(aug->modules, module);
     free((void *) aug->root);
     free(aug->modpathz);
