@@ -22,6 +22,7 @@
 
 #include <config.h>
 #include <argz.h>
+#include <getopt.h>
 
 #include "list.h"
 #include "syntax.h"
@@ -34,7 +35,9 @@ static void usage(void) {
     fprintf(stderr, "Usage: %s [OPTIONS] MODULE\n", progname);
     fprintf(stderr, "Evaluate MODULE. Generally, MODULE should contain unit tests.\n");
     fprintf(stderr, "\nOptions:\n\n");
-    fprintf(stderr, "  -I DIR        Add DIR to the module loadpath. Can be given multiple times.\n");
+    fprintf(stderr, "  -I, --include DIR  search DIR for modules; can be given mutiple times\n");
+    fprintf(stderr, "  --nostdinc         do not search the builtin default directories for modules\n");
+
     exit(EXIT_FAILURE);
 }
 
@@ -43,10 +46,20 @@ int main(int argc, char **argv) {
     struct augeas *aug;
     char *loadpath = NULL;
     size_t loadpathlen = 0;
-
+    enum {
+        VAL_NO_STDINC = CHAR_MAX + 1
+    };
+    struct option options[] = {
+        { "help",      0, 0, 'h' },
+        { "include",   1, 0, 'I' },
+        { "nostdinc",  0, 0, VAL_NO_STDINC },
+        { 0, 0, 0, 0}
+    };
+    int idx;
+    unsigned int flags = AUG_TYPE_CHECK|AUG_NO_DEFAULT_LOAD;
     progname = argv[0];
 
-    while ((opt = getopt(argc, argv, "hI:")) != -1) {
+    while ((opt = getopt_long(argc, argv, "hI:", options, &idx)) != -1) {
         switch(opt) {
         case 'I':
             argz_add(&loadpath, &loadpathlen, optarg);
@@ -54,6 +67,9 @@ int main(int argc, char **argv) {
         case 'h':
             usage();
             break;
+        case VAL_NO_STDINC:
+            flags |= AUG_NO_STDINC;
+            break;
         default:
             usage();
             break;
@@ -66,7 +82,7 @@ int main(int argc, char **argv) {
     }
 
     argz_stringify(loadpath, loadpathlen, PATH_SEP_CHAR);
-    aug = aug_init(NULL, loadpath, AUG_TYPE_CHECK|AUG_NO_DEFAULT_LOAD);
+    aug = aug_init(NULL, loadpath, flags);
     if (__aug_load_module_file(aug, argv[optind]) == -1) {
         fprintf(stderr, "%s: error: Loading failed\n", argv[optind]);
         exit(EXIT_FAILURE);
