@@ -95,7 +95,6 @@ static struct skel *make_skel(struct lens *lens) {
     enum lens_tag tag = lens->tag;
     CALLOC(skel, 1);
     skel->tag = tag;
-    skel->lens = lens;
     return skel;
 }
 
@@ -114,34 +113,6 @@ void free_skel(struct skel *skel) {
     free(skel);
 }
 
-static struct dict *make_dict(char *key,
-                              struct skel *skel, struct dict *subdict) {
-    struct dict *dict;
-    CALLOC(dict, 1);
-    CALLOC(dict->entry, 1);
-    dict->key = key;
-    dict->entry->skel = skel;
-    dict->entry->dict = subdict;
-    dict->mark = dict->entry;
-    return dict;
-}
-
-void free_dict(struct dict *dict) {
-    while (dict != NULL) {
-        struct dict *next = dict->next;
-        while (dict->mark != NULL) {
-            struct dict_entry *del = dict->mark;
-            dict->mark = del->next;
-            free_skel(del->skel);
-            free_dict(del->dict);
-            free(del);
-        }
-        free(dict->key);
-        free(dict);
-        dict = next;
-    }
-}
-
 static void print_skel(struct skel *skel);
 static void print_skel_list(struct skel *skels, const char *beg,
                             const char *sep, const char *end) {
@@ -198,53 +169,6 @@ static void print_dict(struct dict *dict, int indent) {
 }
 #endif
 
-static void dict_append(struct dict **dict, struct dict *d2) {
-    struct dict *d1 = *dict;
-
-    if (d1 == NULL) {
-        *dict = d2;
-        return;
-    }
-
-    struct dict *e2 = d2;
-#ifdef DICT_DUMP
-    printf("DICT_APPEND\n");
-    print_dict(d1, 0);
-    printf("AND\n");
-    print_dict(d2, 0);
-#endif
-    while (e2 != NULL) {
-        struct dict *e1;
-        for (e1=d1; e1 != NULL; e1 = e1->next) {
-            if (e1->key == NULL) {
-                if (e2->key == NULL)
-                    break;
-            } else {
-                if (e2->key != NULL && STREQ(e1->key, e2->key))
-                    break;
-            }
-        }
-        if (e1 == NULL) {
-            struct dict *last = e2;
-            e2 = e2->next;
-            last->next = NULL;
-            list_append(d1, last);
-        } else {
-            struct dict *del = e2;
-            list_append(e1->entry, e2->entry);
-            e2 = e2->next;
-            free(del->key);
-            free(del);
-        }
-    }
-#ifdef DICT_DUMP
-    printf("YIELDS\n");
-    print_dict(d1, 0);
-    printf("END\n");
-#endif
-    *dict = d1;
-}
-
 static void get_expected_error(struct state *state, struct lens *l) {
     char *word, *p, *pat;
 
@@ -267,14 +191,17 @@ static char *token_from_split(struct split *split) {
     return strndup(split->start, split->size);
 }
 
-static void split_append(struct split **split,
-                         const char *text, int start, int end) {
+static struct split *split_append(struct split **split, struct split *tail,
+                                  const char *text, int start, int end) {
     struct split *sp;
-    if (ALLOC(sp) >= 0) {
-        sp->start = text + start;
-        sp->size = end - start;
-        list_append(*split, sp);
-    }
+
+    if (ALLOC(sp) < 0)
+        return NULL;
+
+    sp->start = text + start;
+    sp->size = end - start;
+    list_tail_cons(*split, tail, sp);
+    return tail;
 }
 
 static struct split *next_split(struct state *state) {
@@ -318,52 +245,52 @@ static struct split *split_concat(struct state *state, struct lens *lens) {
     }
 
     int reg = 1;
+    struct split *tail = NULL;
     for (int i=0; i < lens->nchildren; i++) {
         assert(reg < regs.num_regs);
         assert(regs.start[reg] != -1);
-        split_append(&split, outer->start, regs.start[reg], regs.end[reg]);
+        tail = split_append(&split, tail,
+                            outer->start, regs.start[reg], regs.end[reg]);
+        if (tail == NULL)
+            goto error;
         reg += 1 + regexp_nsub(lens->children[i]->ctype);
     }
     assert(reg < regs.num_regs);
+ done:
     free(regs.start);
     free(regs.end);
     return split;
+ error:
+    list_free(split);
+    goto done;
 }
 
 static struct split *split_iter(struct lens *lens, struct split *outer) {
     assert(lens->tag == L_STAR);
 
     int count = 0;
-    struct re_registers regs;
     struct split *split = NULL;
     struct regexp *ctype = lens->child->ctype;
 
-    if (ctype->re != NULL)
-        ctype->re->regs_allocated = REGS_UNALLOCATED;
-    count = regexp_match(ctype, outer->start, outer->size, 0, &regs);
-    if (count == -2) {
-        FIXME("Match failed - produce better error");
-        abort();
-    } else if (count == -1) {
-        return NULL;
-    }
-
-    const int reg = 0;
     int pos = 0;
+    struct split *tail = NULL;
     while (pos < outer->size) {
-        count = regexp_match(ctype, outer->start, outer->size, pos, &regs);
+        count = regexp_match(ctype, outer->start, outer->size, pos, NULL);
         if (count == -2) {
             FIXME("Match failed - produce better error");
             abort();
         } else if (count == -1) {
             break;
         }
-        split_append(&split, outer->start, regs.start[reg], regs.end[reg]);
-        pos = regs.end[reg];
+        tail = split_append(&split, tail, outer->start, pos, pos + count);
+        if (tail == NULL)
+            goto error;
+        pos += count;
     }
-    free(regs.start);
-    free(regs.end);
     return split;
+ error:
+    list_free(split);
+    return NULL;
 }
 
 static int applies(struct lens *lens, struct split *split) {
@@ -586,7 +513,7 @@ static struct skel *parse_concat(struct lens *lens, struct state *state,
 
 static struct tree *get_quant_star(struct lens *lens, struct state *state) {
     assert(lens->tag == L_STAR);
-    struct tree *tree = NULL;
+    struct tree *tree = NULL, *tail = NULL;
     struct split *oldsplit = state->split;
     struct split *split = split_iter(lens, state->split);
 
@@ -594,7 +521,7 @@ static struct tree *get_quant_star(struct lens *lens, struct state *state) {
     while (state->split != NULL) {
         struct tree *t = NULL;
         t = get_lens(lens->child, state);
-        list_append(tree, t);
+        list_tail_cons(tree, tail, t);
         next_split(state);
     }
     if (state->pos != oldsplit->start + oldsplit->size) {
@@ -610,7 +537,7 @@ static struct skel *parse_quant_star(struct lens *lens, struct state *state,
     assert(lens->tag == L_STAR);
     struct split *oldsplit = state->split;
     struct split *split = split_iter(lens, state->split);
-    struct skel *skel = make_skel(lens);
+    struct skel *skel = make_skel(lens), *tail = NULL;
 
     *dict = NULL;
     set_split(state, split);
@@ -618,7 +545,7 @@ static struct skel *parse_quant_star(struct lens *lens, struct state *state,
         struct skel *sk;
         struct dict *di = NULL;
         sk = parse_lens(lens->child, state, &di);
-        list_append(skel->skels, sk);
+        list_tail_cons(skel->skels, tail, sk);
         dict_append(dict, di);
         next_split(state);
     }
@@ -644,12 +571,12 @@ static struct skel *parse_quant_maybe(struct lens *lens, struct state *state,
                                       struct dict **dict) {
     assert(lens->tag == L_MAYBE);
 
-    struct skel *skel = make_skel(lens);
+    struct skel *skel = NULL;
     if (applies(lens->child, state->split)) {
-        struct skel *sk;
-        sk = parse_lens(lens->child, state, dict);
-        list_append(skel->skels, sk);
+        skel = parse_lens(lens->child, state, dict);
     }
+    if (skel == NULL)
+        skel = make_skel(lens);
     return skel;
 }
 
@@ -662,7 +589,7 @@ static struct tree *get_subtree(struct lens *lens, struct state *state) {
     state->value = NULL;
     children = get_lens(lens->child, state);
 
-    tree = make_tree(state->key, state->value, children);
+    tree = make_tree(state->key, state->value, NULL, children);
 
     state->key = key;
     state->value = value;
