@@ -40,37 +40,61 @@
  * Various parameters about env vars, special tree nodes etc.
  */
 
-/* The default location for lens definitions */
+/* Define: AUGEAS_LENS_DIR
+ * The default location for lens definitions */
 #define AUGEAS_LENS_DIR DATADIR "/augeas/lenses"
 
-/* The env var that points to the chroot holding files we may modify.
-   Mostly useful for testing */
+/* Define: AUGEAS_ROOT_ENV
+ * The env var that points to the chroot holding files we may modify.
+ * Mostly useful for testing */
 #define AUGEAS_ROOT_ENV "AUGEAS_ROOT"
 
-/* The root for actual file contents */
+/* Define: AUGEAS_FILES_TREE
+ * The root for actual file contents */
 #define AUGEAS_FILES_TREE "/files"
 
-/* Augeas reports some information in this subtree */
+/* Define: AUGEAS_META_TREE
+ * Augeas reports some information in this subtree */
 #define AUGEAS_META_TREE "/augeas"
 
-/* Information about files */
+/* Define: AUGEAS_META_FILES
+ * Information about files */
 #define AUGEAS_META_FILES AUGEAS_META_TREE AUGEAS_FILES_TREE
 
-/* The root directory */
+/* Define: AUGEAS_META_ROOT
+ * The root directory */
 #define AUGEAS_META_ROOT AUGEAS_META_TREE "/root"
 
-/* How we save files. One of 'backup', 'overwrite' or 'newfile' */
+/* Define: AUGEAS_META_SAVE_MODE
+ * How we save files. One of 'backup', 'overwrite' or 'newfile' */
 #define AUGEAS_META_SAVE_MODE AUGEAS_META_TREE "/save"
 
-/* Name of env var that contains list of paths to search for additional
+/* Define: AUGEAS_CLONE_IF_RENAME_FAILS
+ * Control what save does when renaming the temporary file to its final
+ * destination fails with EXDEV or EBUSY: when this tree node exists, copy
+ * the file contents. If it is not present, simply give up and report an
+ * error.  */
+#define AUGEAS_COPY_IF_RENAME_FAILS \
+    AUGEAS_META_SAVE_MODE "/copy_if_rename_fails"
+
+/* A hierarchy where we record certain 'events', e.g. which tree
+ * nodes actually gotsaved into files */
+#define AUGEAS_EVENTS AUGEAS_META_TREE "/events"
+
+#define AUGEAS_EVENTS_SAVED AUGEAS_EVENTS "/saved"
+
+/* Define: AUGEAS_LENS_ENV
+ * Name of env var that contains list of paths to search for additional
    spec files */
 #define AUGEAS_LENS_ENV "AUGEAS_LENS_LIB"
 
-/* Fairly arbitrary bound on the length of the path we
-   accept from AUGEAS_SPEC_ENV */
+/* Define: MAX_ENV_SIZE
+ * Fairly arbitrary bound on the length of the path we
+ *  accept from AUGEAS_SPEC_ENV */
 #define MAX_ENV_SIZE 4096
 
-/* Character separating paths in a list of paths */
+/* Define: PATH_SEP_CHAR
+ * Character separating paths in a list of paths */
 #define PATH_SEP_CHAR ':'
 
 
@@ -121,6 +145,7 @@
 /* String equality tests, suggested by Jim Meyering. */
 #define STREQ(a,b) (strcmp((a),(b)) == 0)
 #define STRCASEEQ(a,b) (strcasecmp((a),(b)) == 0)
+#define STRCASEEQLEN(a,b,n) (strncasecmp((a),(b),(n)) == 0)
 #define STRNEQ(a,b) (strcmp((a),(b)) != 0)
 #define STRCASENEQ(a,b) (strcasecmp((a),(b)) != 0)
 #define STREQLEN(a,b,n) (strncmp((a),(b),(n)) == 0)
@@ -197,29 +222,38 @@ int pathjoin(char **path, int nseg, ...);
 
 // internal.c
 
-/* Escape nonprintable characters within TEXT, similar to how it's done in
+/* Function: escape
+ * Escape nonprintable characters within TEXT, similar to how it's done in
  * C string literals. Caller must free the returned string.
  */
 char *escape(const char *text, int cnt);
+
+/* Function: unescape */
 char *unescape(const char *s, int len);
+
+/* Function: print_chars */
 int print_chars(FILE *out, const char *text, int cnt);
 
-/* Print a pretty representation of being at position POS within TEXT */
+/* Function: print_pos
+ * Print a pretty representation of being at position POS within TEXT */
 void print_pos(FILE *out, const char *text, int pos);
 char *format_pos(const char *text, int pos);
 
-/* Read the contents of file PATH and return them as one long string. The
+/* Function: read_file
+ * Read the contents of file PATH and return them as one long string. The
  * caller must free the result. Return NULL if any error occurs.
  */
 char* read_file(const char *path);
 
-/* A hidden flag used by augparse to suppress loading of all the modules
-   on the path */
+/* Define: AUG_NO_DEFAULT_LOAD
+ * A hidden flag used by augparse to suppress loading of all the modules
+ * on the path */
 #define AUG_NO_DEFAULT_LOAD (1 << 15)
 
-/* The data structure representing a connection to Augeas. */
+/* Struct: augeas
+ * The data structure representing a connection to Augeas. */
 struct augeas {
-    struct tree      *tree;
+    struct tree      *origin;     /* Actual tree root is origin->children */
     const char       *root;       /* Filesystem root for all files */
                                   /* always ends with '/' */
     unsigned int      flags;      /* Flags passed to AUG_INIT */
@@ -229,34 +263,52 @@ struct augeas {
                                      glibc argz vector */
 };
 
-/* An entry in the global config tree. The data structure allows associating
+/* Struct: tree
+ * An entry in the global config tree. The data structure allows associating
  * values with interior nodes, but the API currently marks that as an error.
+ *
+ * To make dealing with parents uniform, even for the root, we create
+ * standalone trees with a fake root, called origin. That root is generally
+ * not referenced from anywhere. Standalone trees should be created with
+ * MAKE_TREE_ORIGIN.
  */
 struct tree {
     struct tree *next;
+    struct tree *parent;     /* Points to self for root */
     char        *label;      /* Last component of PATH */
     struct tree *children;   /* List of children through NEXT */
     char        *value;
     int          dirty;
 };
 
-/* Allocate a new tree node with the given LABEL, VALUE, and CHILDREN,
+#define ROOT_P(t) ((t) != NULL && (t)->parent == (t)->parent->parent)
+
+/* Function: make_tree
+ * Allocate a new tree node with the given LABEL, VALUE, and CHILDREN,
  * which are not copied. The new tree is marked as dirty
  */
-struct tree *make_tree(char *label, char *value, struct tree *children);
+struct tree *make_tree(char *label, char *value,
+                       struct tree *parent, struct tree *children);
+
+/* Mark a tree as a standalone tree; this creates a fake parent for ROOT,
+ * so that even ROOT has a parent. A new node with only child ROOT is
+ * returned on success, and NULL on failure.
+ */
+struct tree  *make_tree_origin(struct tree *root);
 
 int aug_tree_replace(struct augeas *aug, const char *path, struct tree *sub);
 
-int tree_rm(struct tree **tree, const char *path);
+int tree_rm(struct tree *origin, const char *path);
 struct tree *tree_set(struct tree *tree, const char *path, const char *value);
-int tree_insert(struct tree **tree, const char *path, const char *label,
+int tree_insert(struct tree *origin, const char *path, const char *label,
                 int before);
 int free_tree(struct tree *tree);
 int print_tree(const struct tree *tree, FILE *out, const char *path,
                int pr_hidden);
 int tree_equal(const struct tree *t1, const struct tree *t2);
 
-/* Wrappers to simulate OPEN_MEMSTREAM where that's not available. The
+/* Struct: memstream
+ * Wrappers to simulate OPEN_MEMSTREAM where that's not available. The
  * STREAM member is opened by INIT_MEMSTREAM and closed by
  * CLOSE_MEMSTREAM. The BUF is allocated automatically, but can not be used
  * until after CLOSE_MEMSTREAM has been called. It is the callers
@@ -268,7 +320,8 @@ struct memstream {
     size_t size;
 };
 
-/* Initialize a memstream. On systems that have OPEN_MEMSTREAM, it is used
+/* Function: init_memstream
+ * Initialize a memstream. On systems that have OPEN_MEMSTREAM, it is used
  * to open MS->STREAM. On systems without OPEN_MEMSTREAM, MS->STREAM is
  * backed by a temporary file.
  *
@@ -276,13 +329,28 @@ struct memstream {
  */
 int init_memstream(struct memstream *ms);
 
-/* Close a memstream. After calling this, MS->STREAM can not be used
+/* Function: close_memstream
+ * Close a memstream. After calling this, MS->STREAM can not be used
  * anymore and a string representing whatever was written to it is
  * available in MS->BUF. The caller must free MS->BUF.
  *
  * The caller must free the MEMSTREAM structure.
  */
 int close_memstream(struct memstream *ms);
+
+/*
+ * Path expressions
+ */
+
+struct path;
+
+struct path *make_path(const struct tree *root, const char *path);
+struct tree *path_first(struct path *path);
+struct tree *path_next(struct path *path, struct tree *cur);
+int path_find_one(struct path *path, struct tree **match);
+int path_expand_tree(struct path *path, struct tree **tree);
+void free_path(struct path *path);
+
 #endif
 
 
