@@ -141,6 +141,7 @@ struct value *lns_make_union(struct info *info,
     }
 
     lens = make_lens_binop(L_UNION, info, l1, l2, regexp_union_n);
+    lens->consumes_value = l1->consumes_value && l2->consumes_value;
     return make_lens_value(lens);
 }
 
@@ -162,6 +163,7 @@ struct value *lns_make_concat(struct info *info,
     }
 
     lens = make_lens_binop(L_CONCAT, info, l1, l2, regexp_concat_n);
+    lens->consumes_value = l1->consumes_value || l2->consumes_value;
     return make_lens_value(lens);
 }
 
@@ -276,6 +278,20 @@ struct value *lns_make_prim(enum lens_tag tag, struct info *info,
                                  string->str);
             goto error;
         }
+    } else if (tag == L_DEL) {
+        int cnt;
+        const char *dflt = string->str;
+        cnt = regexp_match(regexp, dflt, strlen(dflt), 0, NULL);
+        if (cnt != strlen(dflt)) {
+            char *s = escape(dflt, -1);
+            char *r = escape(regexp->pattern->str, -1);
+            exn = make_exn_value(info,
+                   "del: the default value '%s' does not match /%s/",
+                   s, r);
+            FREE(s);
+            FREE(r);
+            goto error;
+        }
     }
 
     /* Build the actual lens */
@@ -284,6 +300,7 @@ struct value *lns_make_prim(enum lens_tag tag, struct info *info,
     lens->string = string;
     lens->key = (tag == L_KEY || tag == L_LABEL || tag == L_SEQ);
     lens->value = (tag == L_STORE);
+    lens->consumes_value = (tag == L_STORE);
     lens->atype = regexp_make_empty(info);
     if (tag == L_DEL || tag == L_STORE || tag == L_KEY) {
         lens->ctype = ref(regexp);
@@ -452,21 +469,26 @@ static struct value *typecheck_iter(struct info *info, struct lens *l) {
 
 static struct value *typecheck_maybe(struct info *info, struct lens *l) {
     /* Check (r)? as (<e>|r) where <e> is the empty language */
-    fa_t fa, eps;
     struct value *exn = NULL;
 
-    fa = regexp_to_fa(l->ctype);
-    if (fa == NULL)
-        return make_exn_value(ref(info), "Internal error: regexp_to_fa failed");
-
-    eps = fa_make_basic(FA_EPSILON);
-    if (fa_contains(eps, fa)) {
+    if (regexp_matches_empty(l->ctype)) {
         exn = make_exn_value(ref(info),
                 "illegal optional expression: /%s/ matches the empty word",
                 l->ctype->pattern->str);
     }
-    fa_free(eps);
-    fa_free(fa);
+
+    /* Typecheck the put direction; the check passes if
+       (1) the atype does not match the empty string, because we can tell
+           from looking at tree nodes whether L should be applied or not
+       (2) L handles a value; with that, we know whether to apply L or not
+           depending on whether the current node has a non NULL value or not
+    */
+    if (exn == NULL && ! l->consumes_value) {
+        if (regexp_matches_empty(l->atype)) {
+            exn = make_exn_value(ref(info),
+               "optional expression matches the empty tree but does not consume a value");
+        }
+    }
     return exn;
 }
 
