@@ -48,6 +48,7 @@ struct lens {
     struct regexp            *atype;
     unsigned int              value : 1;
     unsigned int              key : 1;
+    unsigned int              consumes_value : 1;
     union {
         /* Primitive lenses */
         struct {                   /* L_DEL uses both */
@@ -86,29 +87,14 @@ struct value *lns_make_maybe(struct info *, struct lens *,
 /* Auxiliary data structures used during get/put/create */
 struct skel {
     struct skel *next;
-    struct lens *lens;
     enum lens_tag tag;
     union {
         char        *text;    /* L_DEL */
-        struct skel *skels;  /* L_CONCAT, L_STAR, L_MAYBE */
+        struct skel *skels;   /* L_CONCAT, L_STAR */
     };
     /* Also tag == L_SUBTREE, with no data in the union */
 };
 
-/* A dictionary that maps key to a list of (skel, dict) */
-struct dict_entry {
-    struct dict_entry *next;
-    struct skel *skel;
-    struct dict *dict;
-};
-
-struct dict {
-    struct dict *next;
-    char        *key;
-    struct dict_entry *entry; /* This will change as entries are looked up */
-    struct dict_entry *mark;  /* Pointer to initial entry, will never change */
-};
-
 struct lns_error {
     struct lens  *lens;
     int           pos;        /* Errors from get/parse */
@@ -116,6 +102,10 @@ struct lns_error {
     char         *message;
 };
 
+struct dict *make_dict(char *key, struct skel *skel, struct dict *subdict);
+void dict_lookup(const char *key, struct dict *dict,
+                 struct skel **skel, struct dict **subdict);
+int dict_append(struct dict **dict, struct dict *d2);
 void free_skel(struct skel *skel);
 void free_dict(struct dict *dict);
 void free_lns_error(struct lns_error *err);
