@@ -1864,14 +1864,14 @@ yyreduce:
   case 54:
 #line 293 "parser.y"
     {
-               (yyval.tree) = make_tree((yyvsp[(1) - (2)].string), NULL, (yyvsp[(2) - (2)].tree));
+               (yyval.tree) = make_tree((yyvsp[(1) - (2)].string), NULL, NULL, (yyvsp[(2) - (2)].tree));
              }
     break;
 
   case 55:
 #line 297 "parser.y"
     {
-               (yyval.tree) = make_tree((yyvsp[(1) - (4)].string), (yyvsp[(3) - (4)].string), (yyvsp[(4) - (4)].tree));
+               (yyval.tree) = make_tree((yyvsp[(1) - (4)].string), (yyvsp[(3) - (4)].string), NULL, (yyvsp[(4) - (4)].tree));
              }
     break;
 
@@ -2282,7 +2282,7 @@ static struct term *make_test(struct term *test, struct term *result,
 static struct term *make_tree_value(struct tree *tree, struct info *locp) {
   struct term *term = make_term_locp(A_VALUE, locp);
   struct value *value = make_value(V_TREE, ref(term->info));
-  value->tree = tree;
+  value->origin = make_tree_origin(tree);
   term->type = make_base_type(T_TREE);
   term->value = value;
   return term;
@@ -2299,6 +2299,7 @@ void augl_error(struct info *locp,
                 const char *s) {
   struct info info;
   struct string string;
+  MEMZERO(&info, 1);
   info.ref = string.ref = UINT_MAX;
   info.filename = &string;
 
