@@ -137,9 +137,9 @@ static void free_split(struct split *split) {
     free(split);
 }
 
-static void split_append(struct split **split,
-                         struct tree *tree, struct tree *follow,
-                         char *labels, size_t start, size_t end) {
+static struct split *split_append(struct split **split, struct split *tail,
+                                  struct tree *tree, struct tree *follow,
+                                  char *labels, size_t start, size_t end) {
     struct split *sp;
     CALLOC(sp, 1);
     sp->tree = tree;
@@ -147,7 +147,8 @@ static void split_append(struct split **split,
     sp->labels = labels;
     sp->start = start;
     sp->end = end;
-    list_append(*split, sp);
+    list_tail_cons(*split, tail, sp);
+    return tail;
 }
 
 static struct split *next_split(struct state *state) {
@@ -173,9 +174,19 @@ static struct split *split_concat(struct state *state, struct lens *lens) {
     int count = 0;
     struct split *outer = state->split;
     struct re_registers regs;
-    struct split *split = NULL;
+    struct split *split = NULL, *tail = NULL;
     struct regexp *atype = lens->atype;
 
+    /* Fast path for leaf nodes, which will always lead to an empty split */
+    if (outer->tree == NULL && strlen(outer->labels) == 0
+        && regexp_is_empty_pattern(atype)) {
+        for (int i=0; i < lens->nchildren; i++) {
+            tail = split_append(&split, tail, NULL, NULL,
+                                outer->labels, 0, 0);
+        }
+        return split;
+    }
+
     if (atype->re != NULL)
         atype->re->regs_allocated = REGS_UNALLOCATED;
     count = regexp_match(atype, outer->labels, outer->end,
@@ -183,7 +194,7 @@ static struct split *split_concat(struct state *state, struct lens *lens) {
     if (count == -2) {
         FIXME("Match failed - produce better error");
         abort();
-    } else if (count == -1) {
+    } else if (count == -1 || count != outer->end - outer->start) {
         char *labels = strndup(outer->labels + outer->start,
                                outer->end - outer->start);
         put_error(state, lens,
@@ -203,8 +214,8 @@ static struct split *split_concat(struct state *state, struct lens *lens) {
             if (outer->labels[j] == '/')
                 follow = follow->next;
         }
-        split_append(&split, cur, follow,
-                     outer->labels, regs.start[reg], regs.end[reg]);
+        tail = split_append(&split, tail, cur, follow,
+                            outer->labels, regs.start[reg], regs.end[reg]);
         cur = follow;
         reg += 1 + regexp_nsub(lens->children[i]->atype);
     }
@@ -218,27 +229,14 @@ static struct split *split_iter(struct lens *lens, struct split *outer) {
     assert(lens->tag == L_STAR);
 
     int count = 0;
-    struct re_registers regs;
     struct split *split = NULL;
     struct regexp *atype = lens->child->atype;
 
-    if (atype->re != NULL)
-        atype->re->regs_allocated = REGS_UNALLOCATED;
-    count = regexp_match(atype, outer->labels, outer->end,
-                         outer->start, &regs);
-    if (count == -2) {
-        FIXME("Match failed - produce better error");
-        abort();
-    } else if (count == -1) {
-        return NULL;
-    }
-
     struct tree *cur = outer->tree;
-    const int reg = 0;
     int pos = outer->start;
+    struct split *tail = NULL;
     while (pos < outer->end) {
-        count = regexp_match(atype, outer->labels,
-                             outer->end, pos, &regs);
+        count = regexp_match(atype, outer->labels, outer->end, pos, NULL);
         if (count == -2) {
             FIXME("Match failed - produce better error");
             abort();
@@ -246,46 +244,34 @@ static struct split *split_iter(struct lens *lens, struct split *outer) {
             break;
         }
         struct tree *follow = cur;
-        for (int j = regs.start[reg]; j < regs.end[reg]; j++) {
+        for (int j = pos; j < pos + count; j++) {
             if (outer->labels[j] == '/')
                 follow = follow->next;
         }
-        split_append(&split, cur, follow,
-                     outer->labels, regs.start[reg], regs.end[reg]);
+        tail = split_append(&split, tail, cur, follow,
+                            outer->labels, pos, pos + count);
         cur = follow;
-        pos = regs.end[reg];
+        pos += count;
     }
-    free(regs.start);
-    free(regs.end);
     return split;
 }
 
 /* Check if LENS applies to the current split in STATE */
-static int applies(struct lens *lens, struct split *split) {
+static int applies(struct lens *lens, struct state *state) {
     int count;
+    struct split *split = state->split;
+
     count = regexp_match(lens->atype, split->labels, split->end,
                          split->start, NULL);
     if (count == -2) {
         FIXME("Match failed - produce better error");
         abort();
     }
-    return (count == split->end - split->start);
-}
-
-static struct dict_entry *dict_lookup(const char *key, struct dict *dict) {
-    struct dict *d;
-    if (key == NULL) {
-        for (d=dict; d != NULL && d->key != NULL; d = d->next);
-    } else {
-        for (d=dict; d != NULL && (d->key == NULL || STRNEQ(key, d->key));
-             d = d->next);
-    }
-    if (d == NULL)
-        return NULL;
-    struct dict_entry *result = d->entry;
-    if (result != NULL)
-        d->entry = result->next;
-    return result;
+    if (count != split->end - split->start)
+        return 0;
+    if (count == 0 && lens->value)
+        return state->value != NULL;
+    return 1;
 }
 
 /* Print TEXT to OUT, translating common escapes like \n */
@@ -340,16 +326,16 @@ static int skel_instance_of(struct lens *lens, struct skel *skel) {
         return 0;
 
     switch (lens->tag) {
-    case L_DEL:
+    case L_DEL: {
+        int count;
         if (skel->tag != L_DEL)
             return 0;
-        return regexp_match(lens->regexp, skel->text, strlen(skel->text),
-                            0, NULL);
+        count = regexp_match(lens->regexp, skel->text, strlen(skel->text),
+                           0, NULL);
+        return count == strlen(skel->text);
+    }
     case L_STORE:
-        if (skel->tag != L_STORE)
-            return 0;
-        return regexp_match(lens->regexp, skel->text, strlen(skel->text),
-                            0, NULL);
+        return skel->tag == L_STORE;
     case L_KEY:
         return skel->tag == L_KEY;
     case L_LABEL:
@@ -380,8 +366,9 @@ static int skel_instance_of(struct lens *lens, struct skel *skel) {
         break;
     case L_SUBTREE:
         return skel->tag == L_SUBTREE;
-    case L_STAR:
     case L_MAYBE:
+        return skel->tag == L_MAYBE || skel_instance_of(lens->child, skel);
+    case L_STAR:
         if (skel->tag != lens->tag)
             return 0;
         if (lens->tag == L_MAYBE &&
@@ -410,7 +397,6 @@ static void put_subtree(struct lens *lens, struct state *state) {
     size_t oldpathlen = strlen(state->path);
 
     struct tree *tree = state->split->tree;
-    struct dict_entry *entry = dict_lookup(tree->label, state->dict);
     struct split *split = NULL;
 
     state->key = tree->label;
@@ -420,13 +406,10 @@ static void put_subtree(struct lens *lens, struct state *state) {
     split = make_split(tree->children);
     set_split(state, split);
 
-    if (entry == NULL) {
-        state->skel = NULL;
-        state->dict = NULL;
+    dict_lookup(tree->label, state->dict, &state->skel, &state->dict);
+    if (state->skel == NULL || ! skel_instance_of(lens->child, state->skel)) {
         create_lens(lens->child, state);
     } else {
-        state->skel = entry->skel;
-        state->dict = entry->dict;
         put_lens(lens->child, state);
     }
     assert(state->error != NULL || state->split->next == NULL);
@@ -451,7 +434,7 @@ static void put_union(struct lens *lens, struct state *state) {
 
     for (int i=0; i < lens->nchildren; i++) {
         struct lens *l = lens->children[i];
-        if (applies(l, state->split)) {
+        if (applies(l, state)) {
             if (skel_instance_of(l, state->skel))
                 put_lens(l, state);
             else
@@ -459,12 +442,11 @@ static void put_union(struct lens *lens, struct state *state) {
             return;
         }
     }
-    assert(0);
+    put_error(state, lens, "None of the alternatives in the union match");
 }
 
 static void put_concat(struct lens *lens, struct state *state) {
     assert(lens->tag == L_CONCAT);
-    assert(state->skel->lens == lens);
     struct split *oldsplit = state->split;
     struct skel *oldskel = state->skel;
 
@@ -490,7 +472,6 @@ static void put_concat(struct lens *lens, struct state *state) {
 
 static void put_quant_star(struct lens *lens, struct state *state) {
     assert(lens->tag == L_STAR);
-    assert(state->skel->lens == lens);
     struct split *oldsplit = state->split;
     struct skel *oldskel = state->skel;
 
@@ -517,13 +498,13 @@ static void put_quant_star(struct lens *lens, struct state *state) {
 
 static void put_quant_maybe(struct lens *lens, struct state *state) {
     assert(lens->tag == L_MAYBE);
-    assert(state->skel->lens == lens);
+    struct lens *child = lens->child;
 
-    if (applies(lens->child, state->split)) {
-        if (skel_instance_of(lens->child, state->skel))
-            put_lens(lens->child, state);
+    if (applies(child, state)) {
+        if (skel_instance_of(child, state->skel))
+            put_lens(child, state);
         else
-            create_lens(lens->child, state);
+            create_lens(child, state);
     }
 }
 
@@ -601,7 +582,7 @@ static void create_union(struct lens *lens, struct state *state) {
     assert(lens->tag == L_UNION);
 
     for (int i=0; i < lens->nchildren; i++) {
-        if (applies(lens->children[i], state->split)) {
+        if (applies(lens->children[i], state)) {
             create_lens(lens->children[i], state);
             return;
         }
@@ -650,7 +631,7 @@ static void create_quant_star(struct lens *lens, struct state *state) {
 static void create_quant_maybe(struct lens *lens, struct state *state) {
     assert(lens->tag == L_MAYBE);
 
-    if (applies(lens->child, state->split)) {
+    if (applies(lens->child, state)) {
         create_lens(lens->child, state);
     }
 }
