@@ -69,6 +69,14 @@ void free_regexp(struct regexp *regexp) {
     free(regexp);
 }
 
+int regexp_is_empty_pattern(struct regexp *r) {
+    for (char *s = r->pattern->str; *s; s++) {
+        if (*s != '(' && *s != ')')
+            return 0;
+    }
+    return 1;
+}
+
 struct regexp *make_regexp_literal(struct info *info, const char *text) {
     char *pattern, *p;
 
@@ -266,6 +274,10 @@ int regexp_match(struct regexp *r,
     return re_match(r->re, string, size, start, regs);
 }
 
+int regexp_matches_empty(struct regexp *r) {
+    return regexp_match(r, "", 0, 0, NULL) == 0;
+}
+
 int regexp_nsub(struct regexp *r) {
     if (r->re == NULL)
         if (regexp_compile(r) == -1)
