@@ -292,7 +292,7 @@ void free_value(struct value *v) {
         unref(v->lens, lens);
         break;
     case V_TREE:
-        free_tree(v->tree);
+        free_tree(v->origin);
         break;
     case V_FILTER:
         unref(v->filter, filter);
@@ -456,7 +456,7 @@ struct module *module_create(const char *name) {
 
 static struct module *module_find(struct module *module, const char *name) {
     list_for_each(e, module) {
-        if (STREQ(e->name, name))
+        if (STRCASEEQ(e->name, name))
             return e;
     }
     return NULL;
@@ -487,7 +487,7 @@ static struct binding *ctx_lookup_bnd(struct info *info,
         if (dot != NULL) {
         qual_lookup:
             list_for_each(module, ctx->aug->modules) {
-                if (STREQLEN(module->name, name, strlen(module->name))
+                if (STRCASEEQLEN(module->name, name, strlen(module->name))
                     && dot - name == strlen(module->name))
                     return bnd_lookup(module->bindings, dot + 1);
             }
@@ -625,7 +625,7 @@ static void print_value(FILE *out, struct value *v) {
         fprintf(out, ">");
         break;
     case V_TREE:
-        print_tree(v->tree, stdout, "/*" , 1);
+        print_tree(v->origin->children, stdout, "/*" , 1);
         break;
     case V_FILTER:
         fprintf(out, "<filter:");
@@ -681,7 +681,7 @@ static int value_equal(struct value *v1, struct value *v2) {
         return v1->lens == v2->lens;
         break;
     case V_TREE:
-        return tree_equal(v1->tree, v2->tree);
+        return tree_equal(v1->origin->children, v2->origin->children);
         break;
     case V_FILTER:
         return v1->filter == v2->filter;
@@ -1869,6 +1869,8 @@ int interpreter_init(struct augeas *aug) {
     glob_t globbuf;
     int gl_flags = GLOB_NOSORT;
 
+    MEMZERO(&globbuf, 1);
+
     while ((dir = argz_next(aug->modpathz, aug->nmodpath, dir)) != NULL) {
         char *globpat;
         r = asprintf(&globpat, "%s/*.aug", dir);
