@@ -149,6 +149,10 @@ void print_regexp(FILE *out, struct regexp *regexp);
  */
 struct regexp *make_regexp(struct info *info, char *pat);
 
+/* Return 1 if R is an empty pattern, i.e. one consisting of nothing but
+   '(' and ')' characters, 0 otherwise */
+int regexp_is_empty_pattern(struct regexp *r);
+
 /* Make a regexp that matches TEXT literally; the string TEXT
  * is not used by the returned rgexp and must be freed by the caller
  */
@@ -168,6 +172,9 @@ int regexp_compile(struct regexp *r);
 int regexp_match(struct regexp *r, const char *string, const int size,
                  const int start, struct re_registers *regs);
 
+/* Return 1 if R matches the empty string, 0 otherwise */
+int regexp_matches_empty(struct regexp *r);
+
 /* Return the number of subexpressions (parentheses) inside R. May cause
  * compilation of R; return -1 if compilation fails.
  */
@@ -296,7 +303,7 @@ struct value {
         struct regexp  *regexp;  /* V_REGEXP */
         struct lens    *lens;    /* V_LENS */
         struct native  *native;  /* V_NATIVE */
-        struct tree    *tree;    /* V_TREE */
+        struct tree    *origin;  /* V_TREE */
         struct filter  *filter;  /* V_FILTER */
         struct transform *transform; /* V_TRANSFORM */
         struct exn     *exn;     /* V_EXN */
